﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Collections.ObjectModel;

        

namespace BulkMapExample
{
    /// <summary>
    /// Interaction logic for MapControl.xaml
    /// </summary>
    public partial class MapControl : UserControl
    {
        private ManagedProj4Projection m_Projection;
        private ManagedProj4Projection m_Projection2;
        public bool isBing = false;
        public BingMapsMapType bingType;
        Overlay worldOverlay;
        public RectangleShape MapExtent { get; set; }
        public List<LatLon> CoordList;
        public MapControl()
        {

            //canExtend = false;

            InitializeComponent();
    
            
            Map.ExtentOverlay.MouseWheelMode = MapMouseWheelMode.Disabled;
            Map.ExtentOverlay.DoubleLeftClickMode = MapDoubleLeftClickMode.Disabled;
            Map.ExtentOverlay.LeftClickDragMode = MapLeftClickDragMode.Disabled;
            Map.UseLayoutRounding = true;
            m_Projection = new ManagedProj4Projection(ManagedProj4Projection.GetEpsgParametersString(4326), ManagedProj4Projection.GetBingMapParametersString());
            m_Projection2 = new ManagedProj4Projection(ManagedProj4Projection.GetBingMapParametersString(), ManagedProj4Projection.GetEpsgParametersString(4326));
            Map.ExtentOverlay.PanMode = MapPanMode.Disabled;
        }
        public System.Drawing.Bitmap ExportMaptoBmp()
        {
                if (!Map.Overlays.Contains("World"))
                {
                    Map.MapUnit = GeographyUnit.DecimalDegree;
                    Map.ZoomLevelSet = new BingMapsZoomLevelSet();
                    Map.AdornmentOverlay.ShowLogo = false;
                    Map.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(new GeoColor(Colors.LightBlue.R, Colors.LightBlue.G, Colors.LightBlue.B));

                    if (isBing)
                    {

                        BingMapsLayer bingLayer = new BingMapsLayer("ourkey", bingType);
                        bingLayer.LogoMode = BingMapsLogoMode.NoLogo;
                        bingLayer.DrawingExceptionMode = DrawingExceptionMode.ThrowException;
                        bingLayer.TimeoutInSeconds = 0;
                        worldOverlay = new LayerOverlay();
                        ((LayerOverlay)worldOverlay).TransitionEffect = TransitionEffect.None;
                        ((LayerOverlay)worldOverlay).ImageFormat = TileImageFormat.Jpeg;
                        ((LayerOverlay)worldOverlay).JpegQuality = 1;
                        ((LayerOverlay)worldOverlay).TileType = TileType.SingleTile;
                        ((LayerOverlay)worldOverlay).Layers.Add("WorldLayer", bingLayer);
                    }
                    else
                    {
                        ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(System.Windows.Forms.Application.StartupPath+ @"\Shapes\combined.shp");
                        worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.County1;
                        worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                        worldLayer.DrawingQuality = DrawingQuality.HighSpeed;
                        worldOverlay = new LayerOverlay();
                        ((LayerOverlay)worldOverlay).Layers.Add("WorldLayer", worldLayer);
                        BingMapsLayer bml = new BingMapsLayer();

                    }
                    Map.Overlays.Add("World", worldOverlay);

                }
   
                if (!Map.Overlays.Contains("Points"))
                {
                    LayerOverlay pointOverlay = new LayerOverlay();
                    InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
                    if (isBing)
                    {
                        m_Projection.Open();
                        pointLayer.FeatureSource.Projection = m_Projection;
                        m_Projection.Close();
                        pointLayer.ZoomLevelSet = new BingMapsZoomLevelSet();
                    }
                    pointLayer.Name = "PointLayer";
                    pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.StandardColors.DarkRed), 10);
                    pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                    pointLayer.Open();
                    pointLayer.Columns.Add(new FeatureSourceColumn("HYB"));
                    pointOverlay.Layers.Add("PointLayer", pointLayer);
                    pointLayer.Close();
                    Map.Overlays.Add("Points", pointOverlay);
                }



                foreach (InMemoryFeatureLayer layer in ((LayerOverlay)Map.Overlays["Points"]).Layers)
                {
                    layer.InternalFeatures.Clear();

                }
                AddPoints();
                ((LayerOverlay)Map.Overlays["Points"]).Layers["PointLayer"].Open();
                InMemoryFeatureLayer pL = (InMemoryFeatureLayer)Map.FindFeatureLayer("PointLayer");
                if (pL.InternalFeatures.Count > 0)
                {
                    if (pL.InternalFeatures.Count == 1)
                    {
                        Feature feature = pL.InternalFeatures[0];
                        double tempX = feature.GetShape().GetCenterPoint().X;
                        double tempY = feature.GetShape().GetCenterPoint().Y;

                        MapExtent = new RectangleShape(new PointShape(tempX - 3.73, tempY + 2.5), new PointShape(tempX + 3.73, tempY - 2.5));
                    }
                    else
                    {

                        Feature feature = pL.InternalFeatures[0];
                        double tempX = feature.GetShape().GetCenterPoint().X;
                        double tempY = feature.GetShape().GetCenterPoint().Y;

                        RectangleShape staticExtent = new RectangleShape(new PointShape(tempX - 3.73, tempY + 2.5), new PointShape(tempX + 3.73, tempY - 2.5));


                        RectangleShape dynamicExtent = ((LayerOverlay)Map.Overlays["Points"]).Layers["PointLayer"].GetBoundingBox();



                        dynamicExtent = ((LayerOverlay)Map.Overlays["Points"]).Layers["PointLayer"].GetBoundingBox();
                        dynamicExtent = ExtentHelper.ZoomOut(dynamicExtent, 30);

                        if (dynamicExtent.Width < staticExtent.Width && dynamicExtent.Height < staticExtent.Height)
                        {
                            MapExtent = staticExtent;
                        }
                        else
                        {
                            MapExtent = dynamicExtent;
                        }

                    }
 



                }
                else
                    return null;
                List<Layer> layers = new List<Layer>();
                layers.Add(((LayerOverlay)Map.Overlays["World"]).Layers["WorldLayer"]);
                layers.Add(Map.FindFeatureLayer("PointLayer"));
                int w = Convert.ToInt32(Math.Ceiling(this.Width));
                int h = Convert.ToInt32(Math.Ceiling(this.Height));
                System.Drawing.Bitmap nativeImage = new System.Drawing.Bitmap(w, h);
                Collection<SimpleCandidate> simpleCandidates = new Collection<SimpleCandidate>();
                GdiPlusGeoCanvas geoCanvas = new GdiPlusGeoCanvas();
                MapExtent = ExtentHelper.GetDrawingExtent(MapExtent, w, h);
                Map.CurrentExtent = MapExtent;
                if (isBing)
                {
                    geoCanvas.BeginDrawing(nativeImage, MapExtent, GeographyUnit.Meter);
                }
                else
                {
                    geoCanvas.BeginDrawing(nativeImage, MapExtent, GeographyUnit.DecimalDegree);
                }

                foreach (Layer layer in layers)
                {
                    layer.Open();
                    layer.DrawingExceptionMode = DrawingExceptionMode.ThrowException;
                    int tries = 0;
                    while (tries < 20)
                    {
                        try
                        {
                            layer.Draw(geoCanvas, simpleCandidates);

                            break;
                        }
                        catch (Exception exc)
                        {
                            System.Threading.Thread.Sleep(100);
                            tries++;
                        }
                    }
                    if (tries == 5)
                    {
                        lock (typeof(MapControl))
                        {
                            layer.DrawingExceptionMode = DrawingExceptionMode.DrawException;
                            layer.Draw(geoCanvas, simpleCandidates);
                        }
                    }
                }

                if (geoCanvas.IsDrawing)
                {
                    geoCanvas.Flush();
                    geoCanvas.EndDrawing();
                }
                return nativeImage;


            return null;
        }
        public void AddPoints()
        {
            InMemoryFeatureLayer pointLayer = (InMemoryFeatureLayer)Map.FindFeatureLayer("PointLayer");
            foreach (LatLon ll in CoordList)
            {
                PointShape point = new PointShape(ll.lat, ll.lon);
                Feature feature = new Feature(point);
                pointLayer.InternalFeatures.Add(feature);
            }
        }

    }
    public struct LatLon
    {
        public double lat;
        public double lon;
        //public int value;

        public LatLon(double lo, double la)
        {
            lat = la;
            lon = lo;
        }
    }
}


